@@ -5,8 +5,8 @@
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
 #
 # This software was authored by Zhian N. Kamvar and Javier F. Tabima, graduate 
-# students at Oregon State University; and Dr. Nik Grünwald, an employee of 
-# USDA-ARS.
+# students at Oregon State University; Jonah C. Brooks, undergraduate student at
+# Oregon State University; and Dr. Nik Grünwald, an employee of USDA-ARS.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for educational, research and non-profit purposes, without fee, 
@@ -43,10 +43,16 @@
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
 
 .onAttach <- function(...) {
-  startupmsg <- paste("This is poppr version", utils::packageVersion("poppr"))
+  poppr_vers <- utils::packageVersion("poppr")
+  if (length(unlist(poppr_vers)) > 3){
+    appendix <- "\n\nThis version of poppr is under development.\nIf you find any bugs, please report them at https://github.com/grunwaldlab/poppr/issues"
+  } else {
+    appendix <- ""
+  }
+  startupmsg <- paste("This is poppr version", poppr_vers)
   startupmsg <- paste0(startupmsg, ". To get started, type package?poppr")
   paralltype <- ifelse(poppr::poppr_has_parallel(), "available", "unavailable")
-  startupmsg <- paste0(startupmsg, "\nOMP parallel support: ", paralltype)
+  startupmsg <- paste0(startupmsg, "\nOMP parallel support: ", paralltype, appendix)
   packageStartupMessage(startupmsg)
   if (!interactive() || stats::runif(1) > 0.1) return()
 
@@ -59,3 +65,6 @@
   packageStartupMessage(tip)
 }
 
+.onUnload <- function (libpath) {
+  library.dynam.unload("poppr", libpath)
+}
\ No newline at end of file
