@@ -3,6 +3,8 @@ context("plotting tests")
 data("nancycats", package = "adegenet")
 data("Pinf", package = "poppr")
 nancy <- popsub(nancycats, c(1, 9))
+ggversion <- packageVersion("ggplot2")
+oldgg <- package_version("1.0.1")
 
 test_that("info_table plots work", {
 	skip_on_cran()
@@ -64,6 +66,22 @@ test_that("genotype_curve produces boxplots", {
 	expect_output(pg$layers[[3]], "geom_text")
 })
 
+test_that("genotype_curve shows output if not quiet", {
+	skip_on_cran()
+	data("partial_clone", package = "poppr")
+
+	expect_output(genotype_curve(partial_clone, sample = 100), "100%")
+})
+
+test_that("genotype_curve can take less than m-1 loci", {
+  skip_on_cran()
+  nc  <- genotype_curve(nancycats, maxloci = 2, quiet = TRUE)
+  nc1 <- genotype_curve(nancycats, maxloci = 1, quiet = TRUE)
+  expect_equal(ncol(nc), 2L)
+  expect_equal(ncol(nc1), 1L)
+  expect_error(genotype_curve(nancycats[loc = 1]), "at least two loci")
+})
+
 test_that("ia produces histograms", {
 	skip_on_cran()
 	res    <- ia(nancy, sample = 20, valuereturn = TRUE, quiet = TRUE)
@@ -78,15 +96,24 @@ test_that("ia produces histograms", {
 	expect_is(pres, "popprtable")
 	expect_output(pres, "nancy")
 
-
-	expect_output(iaplot$layers[[1]], "geom_histogram")
+	if (ggversion <= oldgg){
+	  expect_output(iaplot$layers[[1]], "geom_histogram")
+	} else {
+	  expect_output(iaplot$layers[[1]], "geom_bar")
+	}
+	
 	expect_output(iaplot$layers[[2]], "geom_rug")
 	expect_output(iaplot$layers[[3]], "geom_vline")
-	expect_output(iaplot$layers[[4]], paste0("geom_text", ".+?", signif(res$index["rbarD"], 3)))
-	expect_output(iaplot$layers[[5]], paste0("geom_text", ".+?", signif(res$index["p.rD"], 3)))
+	expect_output(iaplot$layers[[4]], "geom_text")
+	expect_output(iaplot$layers[[5]], "geom_text")
 	expect_output(iaplot$facet, "facet_null\\(\\)")
 
-	expect_output(poplot$layers[[1]], "geom_histogram")
+	if (ggversion <= oldgg){
+	  expect_output(poplot$layers[[1]], "geom_histogram")
+	} else {
+	  expect_output(poplot$layers[[1]], "geom_bar")
+	}
+	
 	expect_output(poplot$layers[[2]], "geom_rug")
 	expect_output(poplot$layers[[3]], "geom_vline")
 	expect_output(poplot$facet, "facet_wrap\\(population\\)")
@@ -104,8 +131,15 @@ test_that("pair.ia produces a heatmap", {
 	plot(nan.pair, limits = NULL)
 	pplot2 <- ggplot2::last_plot()
 	
-	expect_equivalent(pplot, pplot2)
-	expect_that(pplot2, not(is_equivalent_to(pplot_lim)))
+	if (ggversion <= package_version("1.0.1")){
+	  expect_equivalent(pplot, pplot2)
+	  expect_that(pplot2, not(is_equivalent_to(pplot_lim))) 
+	} else {
+	  expect_equivalent(pplot[-8], pplot2[-8])
+	  expect_that(pplot2[-8], not(is_equivalent_to(pplot_lim[-8])))
+	}
+	
+	
 
 	expect_output(pplot$layers[[1]], "geom_tile")
 	expect_output(pplot$layers[[1]], "stat_identity")
